@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.cpp 676 2008-11-28 18:22:49Z vadim $
+  $Id: FWWindow.cpp 778 2009-03-07 04:32:28Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -59,10 +59,10 @@
 #include "instConf.h"
 #include "instDialog.h"
 #include "HttpGet.h"
+#include "StartTipDialog.h"
 
 #include "fwbuilder/FWReference.h"
 #include "fwbuilder/Policy.h"
-#include "fwbuilder/InterfacePolicy.h"
 #include "fwbuilder/NAT.h"
 #include "fwbuilder/Routing.h"
 #include "fwbuilder/Tools.h"
@@ -151,10 +151,12 @@ using namespace std;
 using namespace Ui;
 
 FWWindow::FWWindow() : QMainWindow(),   // QMainWindow(NULL, Qt::Desktop),
-    m_space(0),
-    instd(0), autosaveTimer(new QTimer(static_cast<QObject*>(this))), 
-    instDialogOnScreenTimer(new QTimer(static_cast<QObject*>(this))), 
-    printer(0), searchObject(0), replaceObject(0)
+                       m_space(0),
+                       instd(0),
+                       instDialogOnScreenTimer(
+                           new QTimer(static_cast<QObject*>(this))), 
+                       printer(0), searchObject(0), replaceObject(0),
+                       auto_load_from_rcs_head_revision(0)
 {
     if (fwbdebug)
     {
@@ -167,6 +169,8 @@ FWWindow::FWWindow() : QMainWindow(),   // QMainWindow(NULL, Qt::Desktop),
     m_mainWindow = new Ui::FWBMainWindow_q();
     m_mainWindow->setupUi(dynamic_cast<QMainWindow*>(this));
 
+    prepareFileOpenRecentMenu();
+
     m_space = new QMdiArea(this);
     setCentralWidget(m_space);
     ProjectPanel *proj = newProjectPanel();
@@ -225,6 +229,90 @@ FWWindow::~FWWindow()
     delete m_mainWindow;
 }
 
+void FWWindow::prepareFileOpenRecentMenu()
+{
+    for (int i = 0; i < MAXRECENTFILES; ++i)
+    {
+        recentFileActs[i] = new QAction(this);
+        recentFileActs[i]->setVisible(false);
+        connect(recentFileActs[i], SIGNAL(triggered()),
+                this, SLOT(openRecentFile()));
+
+        m_mainWindow->menuOpen_Recent->addAction(recentFileActs[i]);
+    }
+    openRecentSeparatorAct = m_mainWindow->menuOpen_Recent->addSeparator();
+    m_mainWindow->menuOpen_Recent->addAction(
+        m_mainWindow->actionClearRecentFiles);
+    updateRecentFileActions();
+}
+
+void FWWindow::clearRecentFilesMenu()
+{
+    QStringList empty_list;
+    st->setRecentFiles(empty_list);
+    updateRecentFileActions();
+}
+
+void FWWindow::updateRecentFileActions()
+{
+    QStringList files = st->getRecentFiles();
+    QMap<QString, int> file_name_counters;
+
+    int numRecentFiles = qMin(files.size(), (int)MAXRECENTFILES);
+
+    for (int i = 0; i < numRecentFiles; ++i)
+    {
+        QString file_name = QFileInfo(files[i]).fileName();
+        int c = file_name_counters[file_name]; // default constructed value is 0
+        file_name_counters[file_name] = c + 1;
+    }
+
+    for (int i = 0; i < numRecentFiles; ++i)
+    {
+        QString file_name = QFileInfo(files[i]).fileName();
+        int c = file_name_counters[file_name];
+        // if c > 1, we have two files with the same name but different path
+        QString text = (c > 1) ? files[i] : file_name;
+        recentFileActs[i]->setText(text);
+        recentFileActs[i]->setData(files[i]);
+        recentFileActs[i]->setVisible(true);
+    }
+    for (int j = numRecentFiles; j < MAXRECENTFILES; ++j)
+        recentFileActs[j]->setVisible(false);
+
+    openRecentSeparatorAct->setVisible(numRecentFiles > 0);
+}
+
+/*
+ * Add file name to the "File/Open Recent" menu.
+ */
+void FWWindow::updateOpenRecentMenu(const QString &fileName)
+{
+    QStringList files = st->getRecentFiles();
+    files.removeAll(fileName);
+    files.prepend(fileName);
+    while (files.size() > MAXRECENTFILES)
+        files.removeLast();
+
+    st->setRecentFiles(files);
+
+    updateRecentFileActions();
+}
+
+void FWWindow::openRecentFile()
+{
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (action)
+        loadFile(action->data().toString(), false);
+}
+
+void FWWindow::registerAutoOpenDocFile(const QString &filename,
+                                       bool load_from_rcs_head)
+{
+    openDocFiles.append(filename);
+    auto_load_from_rcs_head_revision = load_from_rcs_head;
+}
+
 ProjectPanel *FWWindow::newProjectPanel()
 {
     ProjectPanel *projectW = new ProjectPanel(m_space);
@@ -295,10 +383,18 @@ void FWWindow::startupLoad()
         activeProject()->loadState(true);
     }
 
-    for (QStringList::iterator it=openDocFiles.begin(); it!=openDocFiles.end();
-         it++)
+    // foreach is QT macro
+    foreach (QString file, openDocFiles)
     {
-        loadFile(*it);
+        loadFile(file, auto_load_from_rcs_head_revision);
+        updateOpenRecentMenu(file);
+     }
+
+    if (! st->getBool("UI/NoStartTip"))
+    {
+        StartTipDialog *stdlg = new StartTipDialog();
+        stdlg->show();
+        stdlg->raise();
     }
 }
 
@@ -394,12 +490,12 @@ void FWWindow::fileOpen()
     }
 }
 
-void FWWindow::loadFile(const QString &filename)
+void FWWindow::loadFile(const QString &filename, bool load_rcs_head)
 {
     if (activeProject() && activeProject()->getFileName().isEmpty())
     {
         ProjectPanel *proj = activeProject();
-        if (proj->loadFile(filename))
+        if (proj->loadFile(filename, load_rcs_head))
         {
             proj->readyStatus(true);
             proj->loadState(true);
@@ -407,7 +503,7 @@ void FWWindow::loadFile(const QString &filename)
     } else
     {
         std::auto_ptr<ProjectPanel> proj(newProjectPanel());
-        if (proj->loadFile(filename))
+        if (proj->loadFile(filename, load_rcs_head))
         {
             showSub(proj.get());
             proj->readyStatus(true);
@@ -531,8 +627,7 @@ void FWWindow::save()
 
 void FWWindow::loadLibrary(const string &libfpath)
 {
-    if (activeProject())
-        activeProject()->loadLibrary(libfpath);
+    if (activeProject()) activeProject()->loadLibrary(libfpath);
 }
 
 void FWWindow::fileImport()
@@ -922,7 +1017,7 @@ void FWWindow::unlockObject()
 
 void FWWindow::prepareEditMenu()
 {
-    if (!activeProject())  return;
+    if (!activeProject()) return;
 
     bool dupMenuItem=true;
     bool moveMenuItem=true;
@@ -934,8 +1029,9 @@ void FWWindow::prepareEditMenu()
 
     activeProject()->m_panel->om->getMenuState(
         false,
-        dupMenuItem,moveMenuItem,copyMenuItem,pasteMenuItem,
-        delMenuItem,newMenuItem,inDeletedObjects);
+        dupMenuItem, moveMenuItem, copyMenuItem, pasteMenuItem,
+        delMenuItem, newMenuItem, inDeletedObjects
+    );
 
     m_mainWindow->editCopyAction->setEnabled(copyMenuItem);
     m_mainWindow->editDeleteAction->setEnabled(delMenuItem);
@@ -1007,7 +1103,7 @@ void FWWindow::prepareWindowsMenu()
     m_mainWindow->menuWindow->clear();
     QAction * close = m_mainWindow->menuWindow->addAction ("Close");
     QAction * closeAll = m_mainWindow->menuWindow->addAction ("Close All");
-    QAction * title = m_mainWindow->menuWindow->addAction ("Tile");
+    QAction * tile = m_mainWindow->menuWindow->addAction ("Tile");
     QAction * cascade = m_mainWindow->menuWindow->addAction ("Cascade");
     QAction * next = m_mainWindow->menuWindow->addAction ("Next");
     QAction * previous = m_mainWindow->menuWindow->addAction ("Previous");
@@ -1020,7 +1116,7 @@ void FWWindow::prepareWindowsMenu()
     connect(maximize, SIGNAL(triggered()), this, SLOT(maximize()));
     connect(close, SIGNAL(triggered()),m_space, SLOT(closeActiveSubWindow()));
     connect(closeAll, SIGNAL(triggered()),m_space, SLOT(closeAllSubWindows()));
-    connect(title, SIGNAL(triggered()), m_space, SLOT(tileSubWindows()));
+    connect(tile, SIGNAL(triggered()), m_space, SLOT(tileSubWindows()));
     connect(cascade, SIGNAL(triggered()), m_space, SLOT(cascadeSubWindows()));
     connect(next, SIGNAL(triggered()),m_space, SLOT(activateNextSubWindow()));
     connect(previous, SIGNAL(triggered()),m_space, SLOT(activatePreviousSubWindow()));
@@ -1348,14 +1444,12 @@ void FWWindow::updateLastModifiedTimestampForAllFirewalls(FWObject *o)
 
 void FWWindow::updateLastInstalledTimestamp(FWObject *o)
 {
-    if (activeProject())
-        activeProject()->updateLastInstalledTimestamp(o);
+    if (activeProject()) activeProject()->updateLastInstalledTimestamp(o);
 }
 
 void FWWindow::updateLastCompiledTimestamp(FWObject *o)
 {
-    if (activeProject())
-        activeProject()->updateLastCompiledTimestamp(o);
+    if (activeProject()) activeProject()->updateLastCompiledTimestamp(o);
 }
 
 
@@ -1376,34 +1470,29 @@ void FWWindow::delObj(FWObject *obj,bool openobj)
 
 ObjectTreeView* FWWindow::getCurrentObjectTree()
 {
-    if (activeProject())
-        return activeProject()->getCurrentObjectTree();
+    if (activeProject()) return activeProject()->getCurrentObjectTree();
     return 0;
 }
 
 void FWWindow::openObject(QTreeWidgetItem *otvi)
 {
-    if (activeProject())
-        activeProject()->openObject(otvi);
+    if (activeProject()) activeProject()->openObject(otvi);
 }
 
 void FWWindow::openObject(FWObject *obj)
 {
-    if (activeProject())
-        activeProject()->openObject(obj);
+    if (activeProject()) activeProject()->openObject(obj);
 }
 
 bool FWWindow::editObject(FWObject *obj)
 {
-    if (activeProject())
-        return activeProject()->editObject(obj);
+    if (activeProject()) return activeProject()->editObject(obj);
     return false;
 }
 
 void FWWindow::findAllFirewalls (std::list<Firewall *> &fws)
 {
-    if (activeProject())
-        activeProject()->findAllFirewalls (fws);
+    if (activeProject()) activeProject()->findAllFirewalls (fws);
 }
 
 FWObject* FWWindow::duplicateObject(FWObject *target,
@@ -1419,129 +1508,109 @@ FWObject* FWWindow::duplicateObject(FWObject *target,
 
 void FWWindow::showDeletedObjects(bool f)
 {
-    if (activeProject())
-        activeProject()->showDeletedObjects(f);
+    if (activeProject()) activeProject()->showDeletedObjects(f);
 }
 
 void FWWindow::select()
 {
-    if (activeProject())
-        activeProject()->select();
+    if (activeProject()) activeProject()->select();
 }
 
 void FWWindow::unselect()
 {
-    if (activeProject())
-        activeProject()->unselect();
+    if (activeProject()) activeProject()->unselect();
 }
 
 void FWWindow::info()
 {
-    if (activeProject())
-        info(activeProject()->getSelectedObject(), true);
+    if (activeProject()) info(activeProject()->getSelectedObject(), true);
 }
 
 void FWWindow::setManipulatorFocus()
 {
-    if (activeProject())
-        activeProject()->setManipulatorFocus();
+    if (activeProject()) activeProject()->setManipulatorFocus();
 }
 
 void FWWindow::clearManipulatorFocus()
 {
-    if (activeProject())
-        activeProject()->clearManipulatorFocus();
+    if (activeProject()) activeProject()->clearManipulatorFocus();
 }
 
 //wrapers for some Object Editor functions
 bool FWWindow::isEditorVisible()
 {
-    if (activeProject())
-        return activeProject()->isEditorVisible();
+    if (activeProject()) return activeProject()->isEditorVisible();
     return false;
 }
 
 bool FWWindow::isEditorModified()
 {
-    if (activeProject())
-        return activeProject()->isEditorModified();
+    if (activeProject()) return activeProject()->isEditorModified();
     return false;
 }
 
 void FWWindow::showEditor()
 {    
-    if (activeProject())
-        activeProject()->showEditor();
+    if (activeProject()) activeProject()->showEditor();
 }
 
 void FWWindow::hideEditor()
 {
-    if (activeProject())
-        activeProject()->hideEditor();
+    if (activeProject()) activeProject()->hideEditor();
 }
 
 void FWWindow::closeEditor()
 {
-    if (activeProject())
-        activeProject()->closeEditor();
+    if (activeProject()) activeProject()->closeEditor();
 }
 
 void FWWindow::openEditor(FWObject *o)
 {
-    if (activeProject())
-        activeProject()->openEditor(o);
+    if (activeProject()) activeProject()->openEditor(o);
 }
 
 void FWWindow::openOptEditor(FWObject *o, ObjectEditor::OptType t)
 {
-    if (activeProject())
-        activeProject()->openOptEditor(o, t);
+    if (activeProject()) activeProject()->openOptEditor(o, t);
 }
 
 void FWWindow::blankEditor()
 {
-    if (activeProject())
-        activeProject()->blankEditor();
+    if (activeProject()) activeProject()->blankEditor();
 }
 
 
 FWObject* FWWindow::getOpenedEditor()
 {
-    if (activeProject())
-        return activeProject()->getOpenedEditor();
+    if (activeProject()) return activeProject()->getOpenedEditor();
     return 0;
 }
 
 ObjectEditor::OptType FWWindow::getOpenedOptEditor()
 {
-    if (activeProject())
-        return activeProject()->getOpenedOptEditor();
+    if (activeProject()) return activeProject()->getOpenedOptEditor();
     return ObjectEditor::optNone;
 }
 
 void FWWindow::selectObjectInEditor(FWObject *o)
 {
-    if (activeProject())
-        activeProject()->selectObjectInEditor(o);
+    if (activeProject()) activeProject()->selectObjectInEditor(o);
 }
 
 void FWWindow::actionChangedEditor(FWObject *o)
 {
-    if (activeProject())
-        activeProject()->actionChangedEditor(o);
+    if (activeProject()) activeProject()->actionChangedEditor(o);
 }
 
 bool FWWindow::validateAndSaveEditor()
 {
-    if (activeProject())
-        return activeProject()->validateAndSaveEditor();
+    if (activeProject()) return activeProject()->validateAndSaveEditor();
     return false;
 }
 
 void FWWindow::setFDObject(FWObject *o)
 {
-    if (activeProject())
-        activeProject()->setFDObject(o);
+    if (activeProject()) activeProject()->setFDObject(o);
 }
 
 QPrinter* FWWindow::getPrinter()
@@ -1551,32 +1620,19 @@ QPrinter* FWWindow::getPrinter()
 
 FWObjectDatabase* FWWindow::db() 
 { 
-    if (activeProject())
-        return activeProject()->db(); 
+    if (activeProject()) return activeProject()->db(); 
     return 0;
 }
 
 QString FWWindow::printHeader()
 {
-    if (activeProject())
-        return activeProject()->printHeader();
+    if (activeProject()) return activeProject()->printHeader();
     return "";
 }
 
-// NOTE: add-on libraries are obsolete in v3.0.0
-// TODO: remove this.
-listOfLibraries *FWWindow::getAddOnLibs()
-{
-    if (activeProject())
-        return activeProject()->getAddOnLibs();
-    return 0;
-}
-
-
 bool FWWindow::isSystem(FWObject *obj)
 {
-    if (activeProject())
-        return activeProject()->isSystem(obj);
+    if (activeProject()) return activeProject()->isSystem(obj);
     return false;
 }
 
@@ -1585,7 +1641,8 @@ void FWWindow::closeEvent(QCloseEvent* ev)
 {
     if (fwbdebug) qDebug("FWWindow::closeEvent");
 
-    st->setInt("Window/maximized", activeProject()->isMaximized());
+    if (activeProject())
+        st->setInt("Window/maximized", activeProject()->isMaximized());
 
     QList<QMdiSubWindow *> subWindowList = m_space->subWindowList();
     for (int i = 0 ; i < subWindowList.size();i++)
