@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: IPTImporter.cpp 647 2008-11-14 07:03:21Z vadim $
+  $Id: IPTImporter.cpp 672 2008-11-26 17:25:56Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -395,13 +395,13 @@ void IPTImporter::addSrv()
 
 void IPTImporter::pushRule()
 {
-    assert(current_ruleset!=NULL);
+//    assert(current_ruleset!=NULL);
     if (current_rule==NULL) return;
 
     if (current_table=="nat")  pushNATRule();
     else                       pushPolicyRule();
 }
-
+ 
 void IPTImporter::pushPolicyRule()
 {
     // populate all elements of the rule
