@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectEditor.cpp 622 2008-10-20 02:29:48Z vadim $
+  $Id: ObjectEditor.cpp 682 2008-12-04 17:41:44Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -494,8 +494,8 @@ bool ObjectEditor::validateAndSave()
             QMessageBox::warning(
                 dialogs[current_dialog_idx],
                 "Firewall Builder",
-                tr("This object has been modified but not saved.\n"
-                   "Do you want to save it ?"),
+                tr("Object %1 has been modified but not saved.\n"
+                   "Do you want to save it ?").arg(opened->getName().c_str()),
                 tr("&Save"), tr("&Discard"), tr("&Continue editing"),
                 0, 2 ) )
         {
