@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectTreeView.cpp 521 2008-09-10 06:15:27Z vadim $
+  $Id: ObjectTreeView.cpp 607 2008-10-11 22:34:19Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -323,8 +323,8 @@ void ObjectTreeView::updateTreeItems()
             QPixmapCache::insert( icn, pm_obj);
         }
 
-        if (obj->getBool("ro"))  itm->setIcon(0, pm_lock);//setPixmap(0, pm_lock );
-        else                     itm->setIcon(0, pm_obj );
+        if (obj->getRO())  itm->setIcon(0, pm_lock);//setPixmap(0, pm_lock );
+        else               itm->setIcon(0, pm_obj );
 
         Firewall *fw=Firewall::cast(obj);
         if (fw!=NULL)
@@ -832,8 +832,8 @@ void ObjectTreeView::setLockFlags()
             if (lib->getId()!=FWObjectDatabase::STANDARD_LIB_ID &&
                 lib->getId()!=FWObjectDatabase::TEMPLATE_LIB_ID)
             {
-                if (otvi->getFWObject()->getBool("ro"))  Unlockable=true;
-                else                                     Lockable=true;
+                if (otvi->getFWObject()->getRO())  Unlockable = true;
+                else                               Lockable = true;
             }
         }
         ++it;
