@@ -7,7 +7,7 @@
   Author:  alek@codeminders.com
            refactoring and bugfixes: vadim@fwbuilder.org
 
-  $Id: ProjectPanel.cpp 649 2008-11-17 16:45:14Z vadim $
+  $Id: ProjectPanel.cpp 778 2009-03-07 04:32:28Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -706,7 +706,7 @@ void ProjectPanel::updateLastCompiledTimestamp(FWObject *o)
 
 FWObject* ProjectPanel::pasteTo(FWObject *target, FWObject *obj)
 {
-    return m_panel->om->pasteTo(target, obj, true);
+    return m_panel->om->pasteTo(target, obj);
 }
 
 void ProjectPanel::delObj(FWObject *obj,bool openobj)
@@ -1178,13 +1178,6 @@ bool ProjectPanel::requestEditorOwnership(QWidget *w,
     return true;
 }
 
-// NOTE: add-on libraries are obsolete in v3.0.0
-// TODO: remove this.
-listOfLibraries *ProjectPanel::getAddOnLibs()
-{
-    return addOnLibs;
-}
-
 bool ProjectPanel::isSystem(FWObject *obj)
 {
     return objectTreeFormat->isSystem(obj);
@@ -1287,6 +1280,30 @@ void ProjectPanel::closeEvent(QCloseEvent * ev)
     if (fwbdebug) qDebug("ProjectPanel::closeEvent %p title=%s",
                          this, getPageTitle().toAscii().constData());
 
+    // Can't just call fileClose() because I need to ignore event in
+    // case user clicks Cancel in dialog if some data has not been
+    // saved.
+    if (isEditorVisible())
+    {
+        if (!oe->validateAndSave())
+        {
+            ev->ignore();
+            return;
+        }
+        closeEditorPanel();
+    }
+
+    if (!saveIfModified() || !checkin(true))
+    {
+        ev->ignore();
+        return;
+    }
+
+    saveState();
+    fileClose();
+
+#if 0
+
 //    if (!closing)
     saveState();
     //storeLastOpenedLib();
@@ -1311,6 +1328,7 @@ void ProjectPanel::closeEvent(QCloseEvent * ev)
 
     if (fwbdebug)
         qDebug("ProjectPanel::closeEvent main window houskeeping tasks");
+#endif
 
     mw->updateWindowTitle();
 
