@@ -7,7 +7,7 @@
   Author:  alek@codeminders.com
            refactoring and bugfixes: vadim@fwbuilder.org
 
-  $Id: ProjectPanel_file_ops.cpp 660 2008-11-23 03:19:04Z vadim $
+  $Id: ProjectPanel_file_ops.cpp 743 2009-01-19 08:50:59Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -191,46 +191,57 @@ bool ProjectPanel::fileOpen()
         return false;
     }
 
-    return loadFile(fileName);
+    bool res = loadFile(fileName, false);
+    if (res) mw->updateOpenRecentMenu(fileName);
+    return res;
 }
 
-bool ProjectPanel::loadFile(const QString &fileName)
+bool ProjectPanel::loadFile(const QString &fileName, bool load_rcs_head)
 {
-    if (fwbdebug) qDebug("ProjectPanel::loadFile  fileName=%s",
-                         fileName.toLocal8Bit().constData());
+    if (fwbdebug) qDebug("ProjectPanel::loadFile  fileName=%s load_rcs_head=%d",
+                         fileName.toLocal8Bit().constData(), load_rcs_head);
 
     RCSFilePreview  fp(this);
     bool hasRCS = fp.showFileRLog(fileName);
 
-    if ( (!hasRCS) || (fp.exec() == QDialog::Accepted) )
+    // class RCS automatically selected head revision when it opens
+    // file and reads RCS log. User can choose different revision
+    // using RCSFilePreview. If they do, RCSFilePreview::getSelectedRev()
+    // returns selected revision. However if they do not choose, or
+    // dialog is never executed, getSelectedRev() returns head.
+
+    // if load_rcs_head == true, we do not run RCSFilePreview dialog
+    // which meand it will return head revision automatically.
+
+    int dlg_res = QDialog::Accepted;
+    if (hasRCS && !load_rcs_head) dlg_res = fp.exec();
+    if (dlg_res!=QDialog::Accepted) return false;
+
+    if (!saveIfModified() || !checkin(true)) return false;
+    if (!systemFile && rcs!=NULL) fileClose();
+
+    //try to get simple rcs instance from RCS preview
+    RCS *new_rcs = fp.getSelectedRev();
+
+    //if preview cannot give RCS,
+    //get a new RCS from file dialog
+    if (new_rcs==NULL) new_rcs = new RCS(fileName);
+    if (new_rcs==NULL) return false;
+
+    try
     {
-        if (!saveIfModified() || !checkin(true)) return false;
-        if (!systemFile && rcs!=NULL) fileClose();
-
-        //try to get simple rcs instance from RCS preview
-        RCS *new_rcs = fp.getSelectedRev();
-
-        //if preview cannot give RCS,
-        //get a new RCS from file dialog
-        if (new_rcs==NULL) new_rcs = new RCS(fileName);
-        if (new_rcs==NULL) return false;
-
-        try
-        {
-            new_rcs->co();
-        } catch (FWException &ex)
-        {
-            return false;
-        }
-
-        loadFromRCS(new_rcs);
-
-        if (new_rcs->isTemp())
-            unlink(new_rcs->getFileName().toLocal8Bit().constData());
-
-        return true;
+        new_rcs->co();
+    } catch (FWException &ex)
+    {
+        return false;
     }
-    return false;
+
+    loadFromRCS(new_rcs);
+
+    if (new_rcs->isTemp())
+        unlink(new_rcs->getFileName().toLocal8Bit().constData());
+
+    return true;
 }
 
 void ProjectPanel::fileClose()
@@ -262,14 +273,25 @@ void ProjectPanel::fileClose()
     if (fwbdebug) qDebug("ProjectPanel::fileClose(): done");
 }
 
+/*
+ * slot that is called by a timer if user turned on auto-save feature
+ * using controls in the Preferences dialog. Need to save only if data
+ * was modified (flag "dirty" is set).
+ */
+void ProjectPanel::autoSave()
+{
+    if (db() && db()->isDirty() && rcs && !rcs->getFileName().isEmpty())
+        fileSave();
+}
+
 void ProjectPanel::fileSave()
 {
     QStatusBar *sb = mainW->statusBar();
     sb->showMessage( tr("Saving data to file...") );
-    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents,100);
+    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
     save();
     sb->clearMessage();
-    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents,100);
+    QApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
 }
 
 void ProjectPanel::fileSaveAs()
@@ -290,21 +312,19 @@ void ProjectPanel::fileSaveAs()
 
     QString oldFileName = rcs->getFileName();
 
-    QString nfn = chooseNewFileName(
+    QString newFileName = chooseNewFileName(
         oldFileName, tr("Choose name and location for the file"));
 
-    if (!nfn.isEmpty())
+    if (!newFileName.isEmpty())
     {
         db()->setDirty(false);  // so it wont ask if user wants to save
         rcs->abandon();
-
         if (rcs!=NULL) delete rcs;
-
         rcs = new RCS("");
-
-        setFileName(nfn);
-
+        setFileName(newFileName);
         save();
+
+        mw->updateOpenRecentMenu(newFileName);
     }
 }
 
@@ -370,7 +390,7 @@ void ProjectPanel::fileDiscard()
         /* loadFile calls fileClose, but only if file is currently
          * open, which it isn't because we reset rcs to NULL
          */
-        loadFile(fname);
+        loadFile(fname, false);
     }
 }
 
@@ -804,7 +824,7 @@ void ProjectPanel::setupAutoSave()
     {
         int p = st->getInt("Environment/autoSaveFilePeriod");
         autosaveTimer->start( p*1000*60 );
-        connect( autosaveTimer, SIGNAL(timeout()), this, SLOT(fileSave()) );
+        connect( autosaveTimer, SIGNAL(timeout()), this, SLOT(autoSave()) );
     } else
         autosaveTimer->stop();
 }
@@ -1067,7 +1087,7 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
                 qDebug("            filePath: %s",
                        ofinfo.absoluteFilePath().toAscii().constData());
             }
-            QString nfn = ofinfo.dir().absolutePath()
+            QString newFileName = ofinfo.dir().absolutePath()
                 + "/" + ofinfo.completeBaseName() + ".fwb";
 
             bool needToRename = true;
@@ -1076,8 +1096,8 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
              * "Existing .fwb file gets overwritten if has wrong
              * extension"
              */
-            QFileInfo nfinfo(nfn);
-            if (nfinfo.exists() && ofinfo.isSymLink() && ofinfo.readLink()==nfn)
+            QFileInfo nfinfo(newFileName);
+            if (nfinfo.exists() && ofinfo.isSymLink() && ofinfo.readLink()==newFileName)
             {
                 // .xml file is a symlink pointing at .fwb file
                 // no need to rename
@@ -1110,13 +1130,13 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
                            "needs to rename old data file '%1' to '%2',\n"
                            "but file '%3' already exists.\n"
                            "Choose a different name for the new file.")
-                        .arg(fn).arg(nfn).arg(nfn),
+                        .arg(fn).arg(newFileName).arg(newFileName),
                         tr("&Continue"), QString::null,QString::null,
                         0, 1 );
 
-                    nfn = chooseNewFileName(
+                    newFileName = chooseNewFileName(
                         fn, tr("Choose name and location for the new file"));
-                    if (nfn.isEmpty())
+                    if (newFileName.isEmpty())
                     {
                         QString oldFileName = ofinfo.absoluteFilePath()
                             + ".bak";
@@ -1133,22 +1153,22 @@ void ProjectPanel::loadFromRCS(RCS *_rcs)
                         loadStandardObjects();
                         return;
                     }
-                    nfinfo.setFile(nfn);
+                    nfinfo.setFile(newFileName);
                 }
 
                 rename(fn.toLocal8Bit().constData(),
-                       nfn.toLocal8Bit().constData());
+                       newFileName.toLocal8Bit().constData());
 
                 QMessageBox::warning(
                 this,"Firewall Builder",
                 tr("Firewall Builder 2 uses file extension '.fwb'. Your data"
                    "file '%1' \nhas been renamed '%2'")
-                .arg(fn).arg(nfn),
+                .arg(fn).arg(newFileName),
                 tr("&Continue"), QString::null,QString::null,
                 0, 1 );
             }
 
-            fn = nfn;
+            fn = newFileName;
         }
 
         rcs->setFileName(fn);
@@ -1316,7 +1336,7 @@ void ProjectPanel::save()
             else
             {
 /* editingLibfile is true if user edits a library or master library file */
-                bool editingLibfile=editingLibrary();
+                bool editingLibfile = editingLibrary();
 
                 if (st->getDontSaveStdLib())  // this is now default
                 {
