@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleOptionsDialog.h,v 1.5 2006/05/13 06:53:05 vkurland Exp $
+  $Id: RuleOptionsDialog.h 304 2008-07-03 02:19:16Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -28,15 +28,17 @@
 #define __RULEOPTIONSDIALOG_H_
 
 #include "../../config.h"
-#include <ruleoptionsdialog_q.h>
+#include <ui_ruleoptionsdialog_q.h>
+#include <QWidget>
 
 #include "DialogData.h"
 
 #include "fwbuilder/FWObject.h"
 
 class RuleSetView;
+class ProjectPanel;
 
-class RuleOptionsDialog : public RuleOptionsDialog_q
+class RuleOptionsDialog : public QWidget
 {
     Q_OBJECT
 
@@ -44,11 +46,14 @@ class RuleOptionsDialog : public RuleOptionsDialog_q
     DialogData              data;
     QString                 platform;
     RuleSetView            *rsv;
+    Ui::RuleOptionsDialog_q*m_dialog;
     
     bool init;
 
+    ProjectPanel *m_project;
  public:
-    RuleOptionsDialog(QWidget *parent);
+    RuleOptionsDialog(ProjectPanel *project, QWidget *parent);
+    ~RuleOptionsDialog();
     
 public slots:
     virtual void changed();
