@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: platforms.cpp 654 2008-11-20 06:58:15Z vadim $
+  $Id: platforms.cpp 792 2009-03-25 04:43:16Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -147,6 +147,8 @@ void init_platforms()
 
     routeLoadOptions_pf.push_back(QObject::tr("None"));
     routeLoadOptions_pf.push_back("none");
+    routeLoadOptions_pf.push_back(QObject::tr("Bitmask"));
+    routeLoadOptions_pf.push_back("bitmask");
     routeLoadOptions_pf.push_back(QObject::tr("Random"));
     routeLoadOptions_pf.push_back("random");
     routeLoadOptions_pf.push_back(QObject::tr("Source Hash"));
@@ -530,7 +532,8 @@ QString getScreenName(QString s, const QStringList &sl)
  * files and are not pulled using Rule::getActionAsString.
  */
 
-QString getActionNameForPlatform(PolicyRule::Action action,const QString &platform)
+QString getActionNameForPlatform(PolicyRule::Action action,
+                                 const QString &platform)
 {
     QString action_name = "";
     switch (action)
@@ -604,3 +607,39 @@ bool getStatelessFlagForAction(PolicyRule *rule)
     else
         return true;
 }
+
+/**
+ * Returns translatable string - name of the corresponding rule element.
+ */
+QString getReadableRuleElementName(const string &rule_element_type_name)
+{
+    // The following map TYPENAME of RuleElement classes to readable
+    // translatable names.
+    if (rule_element_type_name == "Src") return QObject::tr("Source");
+    if (rule_element_type_name == "Dst") return QObject::tr("Destination");
+    if (rule_element_type_name == "Srv") return QObject::tr("Service");
+    if (rule_element_type_name == "Itf") return QObject::tr("Interface");
+    if (rule_element_type_name == "When") return QObject::tr("Time");
+
+    if (rule_element_type_name == "OSrc") return QObject::tr("Original Src");
+    if (rule_element_type_name == "ODst") return QObject::tr("Original Dst");
+    if (rule_element_type_name == "OSrv") return QObject::tr("Original Srv");
+
+    if (rule_element_type_name == "TSrc") return QObject::tr("Translated Src");
+    if (rule_element_type_name == "TDst") return QObject::tr("Translated Dst");
+    if (rule_element_type_name == "TSrv") return QObject::tr("Translated Srv");
+
+    if (rule_element_type_name == "RDst") return QObject::tr("Destination");
+    if (rule_element_type_name == "RGtw") return QObject::tr("Gateway");
+    if (rule_element_type_name == "RItf") return QObject::tr("Interface");
+
+    // as of v3.0.x the following are not real rule elements (not separate
+    // classes with names) but just attributes of corresponding Rule class.
+    if (rule_element_type_name == "Direction") return QObject::tr("Direction");
+    if (rule_element_type_name == "Action") return QObject::tr("Action");
+    if (rule_element_type_name == "Options") return QObject::tr("Options");
+    if (rule_element_type_name == "Metric") return QObject::tr("Metric");
+    if (rule_element_type_name == "Comment") return QObject::tr("Comment");
+
+    return QString();
+}
