@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: utils.cpp 466 2008-08-13 06:10:55Z vadim $
+  $Id: utils.cpp 748 2009-01-24 07:17:05Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -516,3 +516,14 @@ QString wordWrap(const QString& s, int maxchinline)
     return res;
 }
 
+void loadIcon(QPixmap &pm, libfwbuilder::FWObject *obj)
+{
+    QString icn_file = (":/Icons/" + obj->getTypeName()+"/icon-tree").c_str();
+
+    if ( ! QPixmapCache::find( icn_file, pm) )
+    {
+        pm.load( icn_file );
+        QPixmapCache::insert( icn_file, pm);
+    }
+}
+
