@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_iosacl_writers.cpp 335 2008-07-07 02:32:38Z vadim $
+  $Id: PolicyCompiler_iosacl_writers.cpp 782 2009-03-13 05:04:54Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -30,6 +30,7 @@
 #include "fwbuilder/RuleElement.h"
 #include "fwbuilder/IPService.h"
 #include "fwbuilder/ICMPService.h"
+#include "fwbuilder/ICMP6Service.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
 #include "fwbuilder/CustomService.h"
@@ -136,9 +137,13 @@ bool PolicyCompiler_iosacl::PrintCompleteACLs::processNext()
     PolicyCompiler_iosacl *iosacl_comp=dynamic_cast<PolicyCompiler_iosacl*>(compiler);
     string vers = compiler->fw->getStr("version");
     string platform = compiler->fw->getStr("platform");
+
+    string xml_element = "clear_ip_acl";
+    if (iosacl_comp->ipv6) xml_element = "clear_ipv6_acl";
+
     string clearACLCmd = Resources::platform_res[platform]->getResourceStr(
         string("/FWBuilderResources/Target/options/")+
-        "version_"+vers+"/iosacl_commands/clear_ip_acl");
+        "version_"+vers+"/iosacl_commands/" + xml_element);
 
     assert( !clearACLCmd.empty());
 
@@ -165,8 +170,11 @@ bool PolicyCompiler_iosacl::PrintCompleteACLs::processNext()
     {
         ciscoACL *acl=(*i).second;
         compiler->output << addr_family_prefix
-                         << " access-list extended "
-                         << acl->workName() << endl;
+                         << " access-list ";
+
+        if (!iosacl_comp->ipv6) compiler->output << "extended ";
+
+        compiler->output<< acl->workName() << endl;
         std::for_each(tmp_queue.begin(), tmp_queue.end(),
                       printRulesForACL(iosacl_comp,
                                        this, acl, &(compiler->output)));
@@ -311,7 +319,7 @@ string PolicyCompiler_iosacl::PrintRule::_printLog(PolicyRule *rule)
     return "";
 }
 
-string PolicyCompiler_iosacl::PrintRule::_printSrcService(libfwbuilder::Service *srv)
+string PolicyCompiler_iosacl::PrintRule::_printSrcService(Service *srv)
 {
     ostringstream  str;
 
@@ -333,6 +341,7 @@ string PolicyCompiler_iosacl::PrintRule::_printSrcService(libfwbuilder::Service
 			str << "range " << rs << " " << re << " ";
 	}
     }
+
     return str.str();
 }
 
@@ -386,6 +395,11 @@ string PolicyCompiler_iosacl::PrintRule::_printDstService(Service *srv)
 
     if (ICMPService::isA(srv) && srv->getInt("type")!=-1)
 	    str << srv->getStr("type") << " ";
+
+    if (CustomService::isA(srv)) 
+	str << CustomService::cast(srv)->getCodeForPlatform(
+            compiler->myPlatformName() ) << " ";
+
     return str.str();
 }
 
@@ -397,16 +411,30 @@ string PolicyCompiler_iosacl::PrintRule::_printProtocol(Service *srv)
     if (iosacl_comp->ipv6) addr_family_prefix = "ipv6 ";
 
     string proto = srv->getProtocolName();
+    if (ICMP6Service::isA(srv)) proto = "icmp";
+
+    if (CustomService::isA(srv)) 
+    {
+        // special case standard CusctomService objects "ESTABLISHED"
+        // and "ESTABLISHED ipv6": these require protocol "tcp" but
+        // protocol is set in the Custom Service object for all
+        // platforms at once, so we can't have protocol defined only
+        // for iosacl to be used here.
+	string srv_code = CustomService::cast(srv)->getCodeForPlatform(
+            compiler->myPlatformName());
+        if (srv_code == "established") proto = "tcp";
+    }
+
     if (proto=="ip") return addr_family_prefix;
     return proto + " ";
 }
 
 string PolicyCompiler_iosacl::PrintRule::_printAddr(Address  *o)
 {
-    PolicyCompiler_iosacl *iosacl_comp=dynamic_cast<PolicyCompiler_iosacl*>(compiler);
+    PolicyCompiler_iosacl *iosacl_comp = dynamic_cast<PolicyCompiler_iosacl*>(compiler);
     if (Interface::cast(o)!=NULL)
     {
-	Interface *interface_=Interface::cast(o);
+	Interface *interface_ = Interface::cast(o);
 	if (interface_->isDyn())
         {
 	    return string("interface ") + interface_->getLabel() + " ";
@@ -427,6 +455,7 @@ string PolicyCompiler_iosacl::PrintRule::_printAddr(Address  *o)
         {
             if (Interface::cast(o)==NULL &&
                 Interface::cast(o->getParent())==NULL &&
+                o->dimension() > 1 &&
                 !srcmask.isHostMask())
             {
                 if (iosacl_comp->ipv6)
