@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_PrintRule.cpp 617 2008-10-18 18:45:02Z vadim $
+  $Id: NATCompiler_PrintRule.cpp 712 2008-12-28 07:12:12Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -32,6 +32,7 @@
 #include "fwbuilder/ICMPService.h"
 #include "fwbuilder/TCPService.h"
 #include "fwbuilder/UDPService.h"
+#include "fwbuilder/UserService.h"
 #include "fwbuilder/CustomService.h"
 #include "fwbuilder/TagService.h"
 #include "fwbuilder/Policy.h"
@@ -96,7 +97,8 @@ string NATCompiler_ipt::PrintRule::_createChain(const string &chain)
 
     if ( ipt_comp->minus_n_commands->count(chain)==0 )
     {
-	res << "$IPTABLES -t nat -N " << chain << endl;
+        string ipt_cmd = (ipt_comp->ipv6) ? "$IP6TABLES " : "$IPTABLES ";
+	res << ipt_cmd << "-t nat -N " << chain << endl;
 	(*(ipt_comp->minus_n_commands))[chain] = true;
     }
     return res.str();
@@ -104,7 +106,9 @@ string NATCompiler_ipt::PrintRule::_createChain(const string &chain)
 
 string NATCompiler_ipt::PrintRule::_startRuleLine()
 {            
-    return string("$IPTABLES -t nat -A ");
+    NATCompiler_ipt *ipt_comp = dynamic_cast<NATCompiler_ipt*>(compiler);
+    string res = (ipt_comp->ipv6) ? "$IP6TABLES " : "$IPTABLES ";
+    return res + string("-t nat -A ");
 }
 
 string NATCompiler_ipt::PrintRule::_endRuleLine()
@@ -192,14 +196,27 @@ string NATCompiler_ipt::PrintRule::_printChainDirectionAndInterface(NATRule *rul
     return ostr.str();
 }
 
-
-
 string NATCompiler_ipt::PrintRule::_printProtocol(Service *srv)
 {
     std::ostringstream  ostr;
-    if (!srv->isAny() && !CustomService::isA(srv) && !TagService::isA(srv))
+    // CustomService returns protocol name starting with v3.0.4
+    // However CustomService can return protocol name "any", which we should
+    // just skip.
+    if (CustomService::isA(srv))
     {
-        string pn=srv->getProtocolName();
+        // check if the code string for this custom service already includes
+        // "-p proto" fragment
+        string code = CustomService::cast(srv)->getCodeForPlatform(
+            compiler->myPlatformName());
+        std::size_t minus_p = code.find("-p ");
+        if (minus_p != string::npos) return "";
+        string pn = srv->getProtocolName();
+        if (pn == "any") return "";
+    }
+
+    if (!srv->isAny() && !TagService::isA(srv) && !UserService::isA(srv))
+    {
+        string pn = srv->getProtocolName();
         if (pn=="ip") pn="all";
         ostr << "-p " <<  pn << " ";
         if (pn == "tcp")  ostr << "-m tcp ";
