@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_PrintRule.cpp 657 2008-11-21 06:40:07Z vadim $
+  $Id: PolicyCompiler_PrintRule.cpp 751 2009-01-25 06:35:06Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -37,6 +37,7 @@
 #include "fwbuilder/Policy.h"
 #include "fwbuilder/Network.h"
 #include "fwbuilder/DNSName.h"
+#include "fwbuilder/AddressRange.h"
 
 #include "fwbuilder/FWObjectDatabase.h"
 #include "fwbuilder/RuleElement.h"
@@ -176,8 +177,16 @@ string PolicyCompiler_ipt::PrintRule::_printRuleLabel(PolicyRule *rule)
  */
 string PolicyCompiler_ipt::PrintRule::_printChain(PolicyRule *rule)
 {
-    string s=rule->getStr("ipt_chain");
-    if (s.empty()) s="UNKNOWN";
+    string s = rule->getStr("ipt_chain");
+    if (s.empty()) s = "UNKNOWN";
+    // check chain name length per bug report #2507239
+    if (s.length() > 30)
+    {
+        ostringstream str;
+        str << "Chain name '" << s << "' ";
+        str << "is longer than 30 characters. Rule " << rule->getLabel();
+        compiler->abort(str.str());
+    }
     s= s + " ";
     return s;
 }
@@ -432,17 +441,18 @@ string PolicyCompiler_ipt::PrintRule::_printDirectionAndInterface(PolicyRule *ru
     return ostr.str();
 }
 
-string PolicyCompiler_ipt::PrintRule::_printActionOnReject(libfwbuilder::PolicyRule *rule)
+string PolicyCompiler_ipt::PrintRule::_printActionOnReject(PolicyRule *rule)
 {
     std::ostringstream str;
 
-    PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
+    PolicyCompiler_ipt *ipt_comp = dynamic_cast<PolicyCompiler_ipt*>(compiler);
 
 //    RuleElementSrv *srvrel=rule->getSrv();
-    Service        *srv   =compiler->getFirstSrv(rule);  assert(srv);
+    Service *srv = compiler->getFirstSrv(rule);
+    assert(srv);
 
-    string version=compiler->fw->getStr("version");
-    string s=ipt_comp->getActionOnReject(rule);
+    string version = compiler->fw->getStr("version");
+    string s = ipt_comp->getActionOnReject(rule);
     if (!s.empty()) 
     {
         if (ipt_comp->isActionOnRejectTCPRST(rule))
@@ -671,16 +681,30 @@ string PolicyCompiler_ipt::PrintRule::_printLimit(libfwbuilder::PolicyRule *rule
     return str.str();
 }
 
-string PolicyCompiler_ipt::PrintRule::_printProtocol(libfwbuilder::Service *srv)
+string PolicyCompiler_ipt::PrintRule::_printProtocol(Service *srv)
 {
     PolicyCompiler_ipt *ipt_comp = dynamic_cast<PolicyCompiler_ipt*>(compiler);
     string version = compiler->fw->getStr("version");
     string s;
-    if (! srv->isAny() && !CustomService::isA(srv) &&
-        !TagService::isA(srv) && !UserService::isA(srv))
+    // CustomService returns protocol name starting with v3.0.4
+    // However CustomService can return protocol name "any", which we should
+    // just skip.
+    if (CustomService::isA(srv))
+    {
+        // check if the code string for this custom service already includes
+        // "-p proto" fragment
+        string code = CustomService::cast(srv)->getCodeForPlatform(
+            compiler->myPlatformName());
+        std::size_t minus_p = code.find("-p ");
+        if (minus_p != string::npos) return "";
+        string pn = srv->getProtocolName();
+        if (pn == "any") return "";
+    }
+
+    if (!srv->isAny() && !TagService::isA(srv) && !UserService::isA(srv))
     {
         string pn = srv->getProtocolName();
-        if (pn=="ip") pn = "all";
+        if (pn=="ip" || pn=="any") pn = "all";
 
         if (ipt_comp->ipv6)
         { 
@@ -750,8 +774,8 @@ string PolicyCompiler_ipt::PrintRule::_printSrcPorts(Service *srv)
     std::ostringstream  str;
     if (TCPService::isA(srv) || UDPService::isA(srv)) 
     {
-	int rs=TCPUDPService::cast(srv)->getSrcRangeStart();
-	int re=TCPUDPService::cast(srv)->getSrcRangeEnd();
+	int rs = TCPUDPService::cast(srv)->getSrcRangeStart();
+	int re = TCPUDPService::cast(srv)->getSrcRangeEnd();
 	str << _printPorts(rs,re);
     }
     return str.str();
@@ -762,8 +786,8 @@ string PolicyCompiler_ipt::PrintRule::_printDstPorts(Service *srv)
     std::ostringstream  str;
     if (TCPService::isA(srv) || UDPService::isA(srv)) 
     {
-	int rs=TCPUDPService::cast(srv)->getDstRangeStart();
-	int re=TCPUDPService::cast(srv)->getDstRangeEnd();
+	int rs = TCPUDPService::cast(srv)->getDstRangeStart();
+	int re = TCPUDPService::cast(srv)->getDstRangeEnd();
 	str << _printPorts(rs,re);
     }
     return str.str();
@@ -1052,6 +1076,46 @@ string PolicyCompiler_ipt::PrintRule::_printDstService(RuleElementSrv  *rel)
     return ostr.str();
 }
 
+string PolicyCompiler_ipt::PrintRule::_printSrcAddr(RuleElement *rel,
+                                                    Address  *o)
+{
+    string res;
+    if (AddressRange::cast(o)!=NULL)
+    {
+        AddressRange *ar = AddressRange::cast(o);
+        const InetAddr &range_start = ar->getRangeStart();
+        const InetAddr &range_end = ar->getRangeEnd();
+        if (range_start != range_end)
+        {
+            if (!have_m_iprange) { res = "-m iprange "; have_m_iprange = true; }
+            res += _printSingleObjectNegation(rel) + "--src-range ";
+            res += range_start.toString() + "-" + range_end.toString() + " ";
+            return res;
+        }
+    }
+    return string(" -s ") + _printSingleObjectNegation(rel) + _printAddr(o);
+}
+
+string PolicyCompiler_ipt::PrintRule::_printDstAddr(RuleElement *rel,
+                                                    Address  *o)
+{
+    string res;
+    if (AddressRange::cast(o)!=NULL)
+    {
+        AddressRange *ar = AddressRange::cast(o);
+        const InetAddr &range_start = ar->getRangeStart();
+        const InetAddr &range_end = ar->getRangeEnd();
+        if (range_start != range_end)
+        {
+            if (!have_m_iprange) { res = "-m iprange "; have_m_iprange = true; }
+            res += _printSingleObjectNegation(rel) + "--dst-range ";
+            res += range_start.toString() + "-" + range_end.toString() + " ";
+            return res;
+        }
+    }
+    return string(" -d ") + _printSingleObjectNegation(rel) + _printAddr(o);
+}
+
 string PolicyCompiler_ipt::PrintRule::_printAddr(Address  *o)
 {
     PolicyCompiler_ipt *ipt_comp=dynamic_cast<PolicyCompiler_ipt*>(compiler);
@@ -1098,7 +1162,8 @@ string PolicyCompiler_ipt::PrintRule::_printAddr(Address  *o)
         return ostr.str();
     }
 
-    if (addr->isAny() && mask->isAny())
+    // Note that mask can be NULL, for example if o is AddressRange.
+    if (addr->isAny() && (mask==NULL || mask->isAny()))
     {
         ostr << "0/0 ";
     } else 
@@ -1321,6 +1386,8 @@ string PolicyCompiler_ipt::PrintRule::PolicyRuleToString(PolicyRule *rule)
 
     std::ostringstream  command_line;
 
+    have_m_iprange = false;
+
     command_line << _startRuleLine();
 
     command_line << _printChain(rule);
@@ -1371,23 +1438,17 @@ string PolicyCompiler_ipt::PrintRule::PolicyRuleToString(PolicyRule *rule)
  *
  */
             if (src->hasInetAddress() && !src->getAddressPtr()->isAny())
-            {
-                command_line << " -s " << _printSingleObjectNegation(srcrel);
-                command_line << _printAddr(src);
-            }
+                command_line << _printSrcAddr(srcrel, src);
+
         } else
-        {
-            command_line << " -s " << _printSingleObjectNegation(srcrel);
-            command_line << _printAddr(src);
-        }
+            command_line << _printSrcAddr(srcrel, src);
+
     }
     command_line << _printSrcService(srvrel);
 
     if (!dst->isAny()) 
-    {
-	command_line << " -d " << _printSingleObjectNegation(dstrel);
-	command_line << _printAddr(dst);
-    }
+	command_line << _printDstAddr(dstrel, dst);
+
     command_line << _printDstService(srvrel);
 
 /* keeping state does not apply to deny/reject 
@@ -1486,19 +1547,44 @@ string PolicyCompiler_ipt::PrintRule::_commit()
 
 string PolicyCompiler_ipt::PrintRule::_clampTcpToMssRule()
 {
+    PolicyCompiler_ipt *ipt_comp = dynamic_cast<PolicyCompiler_ipt*>(compiler);
     ostringstream res;
-    bool ipforward = false;
-    string s = compiler->getCachedFwOpt()->getStr("linux24_ip_forward");
-    ipforward= (s.empty() || s=="1" || s=="On" || s=="on");
 
-    if ( compiler->getCachedFwOpt()->getBool("clamp_mss_to_mtu") &&
-         ipforward)
+    if ( compiler->getCachedFwOpt()->getBool("clamp_mss_to_mtu"))
     {
-        res << _startRuleLine()
-            << "FORWARD -p tcp -m tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu"
-            << _endRuleLine();
-        
-        res << endl;
+        bool add_rule = false;
+        bool ipforw;
+        if (ipt_comp->ipv6)
+        {
+            string s = compiler->getCachedFwOpt()->getStr("linux24_ipv6_forward");
+            ipforw = (s.empty() || s=="1" || s=="On" || s=="on");
+            // bug #2477775: target TCPMSS is not available in ip6tables
+            // before 1.4.0 In fact I am not sure of the minimal required
+            // version. According to the netfilter git log, it was added in
+            // 1.3.8
+            string version = compiler->fw->getStr("version");
+            if (XMLTools::version_compare(version, "1.3.8")<0)
+            {
+                if (ipforw)
+                {
+                    res << "target TCPMSS is not supported by ip6tables before v1.3.8";
+                    compiler->warning(res.str());
+                    return "# " + res.str() + "\n\n";
+                } else return "";
+            }
+        } else
+        {
+            string s = compiler->getCachedFwOpt()->getStr("linux24_ip_forward");
+            ipforw = (s.empty() || s=="1" || s=="On" || s=="on");
+        }
+
+        if (ipforw)
+        {
+            res << _startRuleLine()
+                << "FORWARD -p tcp -m tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu"
+                << _endRuleLine();
+            res << endl;
+        }
     }
     return res.str();
 }
@@ -1514,9 +1600,12 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
  * Need rules in FORWARD chain only if ip forwarding is on or set to
  * "no change"
  */
-    bool ipforward = false;
     string s = compiler->getCachedFwOpt()->getStr("linux24_ip_forward");
-    ipforward= (s.empty() || s=="1" || s=="On" || s=="on");
+    bool ipforward= (s.empty() || s=="1" || s=="On" || s=="on");
+    s = compiler->getCachedFwOpt()->getStr("linux24_ipv6_forward");
+    bool ip6forward= (s.empty() || s=="1" || s=="On" || s=="on");
+    bool ipforw = ((!ipt_comp->ipv6 && ipforward) ||
+                   (ipt_comp->ipv6 && ip6forward));
 
     if ( compiler->getCachedFwOpt()->getBool("accept_established") &&
          ipt_comp->my_table=="filter") 
@@ -1529,7 +1618,7 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
             << "OUTPUT  -m state --state ESTABLISHED,RELATED -j ACCEPT"
             << _endRuleLine();
 
-        if (ipforward)
+        if (ipforw)
             res << _startRuleLine()
                 << "FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT"
                 << _endRuleLine();
@@ -1611,7 +1700,7 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
             << "OUTPUT  -p tcp -m tcp ! --tcp-flags SYN,RST,ACK SYN -m state --state NEW -j DROP"
             << _endRuleLine();
 
-        if (ipforward)
+        if (ipforw)
             res << _startRuleLine()
                 << "FORWARD -p tcp -m tcp ! --tcp-flags SYN,RST,ACK SYN -m state --state NEW -j DROP"
                 << _endRuleLine();
@@ -1635,7 +1724,7 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
                 << "INPUT    -m state --state INVALID  -j DROP"
                 << _endRuleLine();
 
-            if (ipforward)
+            if (ipforw)
                 res << _startRuleLine()
                     << "FORWARD  -m state --state INVALID  -j DROP"
                     << _endRuleLine();
@@ -1651,7 +1740,7 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
                 << "INPUT    -m state --state INVALID  -j drop_invalid"
                 << _endRuleLine();
 
-            if (ipforward)
+            if (ipforw)
                 res << _startRuleLine()
                     << "FORWARD  -m state --state INVALID  -j drop_invalid"
                     << _endRuleLine();
