@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt_optimizer.cpp 633 2008-10-25 17:31:30Z vadim $
+  $Id: PolicyCompiler_ipt_optimizer.cpp 709 2008-12-25 20:38:53Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -71,7 +71,7 @@ void PolicyCompiler_ipt::optimize1::optimizeForRuleElement(
     string this_chain = rule->getStr("ipt_chain");
     string new_chain  = ipt_comp->getNewTmpChainName(rule);
 
-    r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+    r= compiler->dbcopy->createPolicyRule();
     compiler->temp_ruleset->add(r);
     r->duplicate(rule);
 
