@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: ipt.cpp 674 2008-11-28 18:05:31Z vadim $
+  $Id: ipt.cpp 754 2009-02-01 09:30:49Z svn $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -112,6 +112,7 @@ static bool             ipv6_run       = true;
 static bool             fw_by_id       = false;
 
 FWObjectDatabase *objdb = NULL;
+bool prolog_done = false;
 
 static map<string,RuleSet*> branches;
 
@@ -133,35 +134,11 @@ class UpgradePredicate: public XMLTools::UpgradePredicate
     public:
     virtual bool operator()(const string&) const 
     { 
-	cout << _("Data file has been created in the old version of Firewall Builder. Use fwbuilder GUI to convert it.") << std::endl;
+	cout << "Data file has been created in the old version of Firewall Builder. Use fwbuilder GUI to convert it." << std::endl;
 	return false;
     }
 };
 
-string addPrologScript(bool nocomment,const string &script)
-{
-    string res="";
-
-    if ( !nocomment )
-    {
-        res += "\n";
-        res += "#\n";
-        res += "# Prolog script\n";
-        res += "#\n";
-    }
-
-    res += script;
-    res += "\n";
-
-    if ( !nocomment )
-    {
-        res += "#\n";
-        res += "# End of prolog script\n";
-        res += "#\n";
-    }
-    return res;
-}
-
 void assignRuleSetChain(RuleSet *ruleset)
 {
     string branch_name = ruleset->getName();
@@ -212,7 +189,18 @@ void findBranchesInMangleTable(Firewall*, list<FWObject*> &all_policies)
 }
 
 /* Find rulesets that belong to other firewall objects but are
- * referenced by rules of this firewall using action Branch
+ * referenced by rules of this firewall using action Branch.
+ *
+ * Important: rulesets that belong to other firewalls may be marked as
+ * "top rulesets", which means they should be translated into the
+ * built-in chains INPUT/OUTPUT/FORWARD rather then into named chain
+ * with the name the same as the name of the ruleset. However this
+ * does not make sense if we want to jump to that ruleset from a rule
+ * from a ruleset that belongs to the firewall we are compiling. If we
+ * compile such "foreighn" ruleset as "top ruleset", then we do not
+ * create chain we would jump to. To avoid this will reset "top
+ * ruleset" flag of rulesets of other firewalls referenced by
+ * branching rules of the firewall being compiled.
  */
 void findImportedRuleSets(Firewall *fw, list<FWObject*> &all_policies)
 {
@@ -229,6 +217,7 @@ void findImportedRuleSets(Firewall *fw, list<FWObject*> &all_policies)
                 (ruleset = rule->getBranch())!=NULL &&
                 !ruleset->isChildOf(fw))
             {
+                ruleset->setTop(false);
                 imported_policies.push_back(ruleset);
             }
         }
@@ -247,18 +236,12 @@ string dumpScript(bool nocomm, Firewall *fw,
 {
     ostringstream res;
     ostringstream script;
-    string prolog_place= fw->getOptionsObject()->getStr("prolog_place");
+    string prolog_place = fw->getOptionsObject()->getStr("prolog_place");
 
     if (fw->getOptionsObject()->getBool("use_iptables_restore"))
     {
         script << reset_script;
 
-        if (prolog_place == "after_flush")
-        {
-            script << addPrologScript(
-                nocomm, fw->getOptionsObject()->getStr("prolog_script"));
-        }
-
         if (!filter_script.empty())
         {
             script << filter_script;
@@ -283,23 +266,21 @@ string dumpScript(bool nocomm, Firewall *fw,
             res << script.str();
             res << "#" << "\n";
             if (ipv6_policy)
-                res << ") | $IP6TABLES_RESTORE; IPTABLES_RESTORE_RES=$?"
-                    << "\n";
-            else
-                res << ") | $IPTABLES_RESTORE; IPTABLES_RESTORE_RES=$?"
-                    << "\n";
+            {
+                res << ") | $IP6TABLES_RESTORE; IPTABLES_RESTORE_RES=$?\n";
+                res << "test $IPTABLES_RESTORE_RES != 0 && ";
+                res << "run_epilog_and_exit $IPTABLES_RESTORE_RES\n";
+            } else
+            {
+                res << ") | $IPTABLES_RESTORE; IPTABLES_RESTORE_RES=$?\n";
+                res << "test $IPTABLES_RESTORE_RES != 0 && ";
+                res << "run_epilog_and_exit $IPTABLES_RESTORE_RES\n";
+            }
         }
         return res.str();
     } else
     {
         res << reset_script;
-
-        if (prolog_place == "after_flush")
-        {
-            res << addPrologScript(
-                nocomm, fw->getOptionsObject()->getStr("prolog_script"));
-        }
-
         if (!nat_script.empty()) res << nat_script;
         if (!mangle_script.empty()) res << mangle_script;
         if (!filter_script.empty())  res << filter_script;
@@ -309,27 +290,31 @@ string dumpScript(bool nocomm, Firewall *fw,
 }
  
 bool processPolicyRuleSet(
-    const QString &fwobjectname,
+    Firewall *fw,
     FWObject *ruleset,
     ostringstream &filter_table_stream,
     ostringstream &mangle_table_stream,
-    ostringstream &reset_rules_stream,
+    ostringstream &automatic_rules_stream,
     OSConfigurator_linux24 *oscnf,
-    bool ipv6_policy,
+    int policy_af,
     std::map<const std::string, bool> &minus_n_commands_filter,
     std::map<const std::string, bool> &minus_n_commands_mangle)
 {
+    const QString &fwobjectname = fw->getName().c_str();
     int policy_rules_count  = 0;
     int mangle_rules_count  = 0;
     bool have_connmark = false;
     bool have_connmark_in_output = false;
     bool empty_output = true;
+    string prolog_place = fw->getOptionsObject()->getStr("prolog_place");
 
     Policy *policy = Policy::cast(ruleset);
     assignRuleSetChain(policy);
     string branch_name = policy->getName();
 
-    if (policy->isV6()!=ipv6_policy) return true;
+    if (!policy->matchingAddressFamily(policy_af)) return true;
+
+    bool ipv6_policy = (policy_af == AF_INET6);
 
     MangleTableCompiler_ipt m(
         objdb , fwobjectname.toUtf8().constData(),
@@ -375,12 +360,13 @@ bool processPolicyRuleSet(
             mangle_table_stream << "automatic rules";
             mangle_table_stream << "\n";
             mangle_table_stream << m.flushAndSetDefaultPolicy();
+            mangle_table_stream << m.printAutomaticRules();
         }
 
         if (m.getCompiledScriptLength() > 0)
         {
-            mangle_table_stream << "# ================ Table 'mangle', rule set "
-                  << branch_name << "\n";
+            mangle_table_stream << "# ================ Table 'mangle', ";
+            mangle_table_stream << "rule set " << branch_name << "\n";
             if (m.haveErrorsAndWarnings())
             {
                 mangle_table_stream << "# Policy compiler errors and warnings:"
@@ -422,8 +408,8 @@ bool processPolicyRuleSet(
 
         if (c.getCompiledScriptLength() > 0)
         {
-            filter_table_stream << "# ================ Table 'filter', rule set "
-                  << branch_name << "\n";
+            filter_table_stream << "# ================ Table 'filter', ";
+            filter_table_stream << "rule set " << branch_name << "\n";
             if (c.haveErrorsAndWarnings())
             {
                 filter_table_stream << "# Policy compiler errors and warnings:"
@@ -437,12 +423,31 @@ bool processPolicyRuleSet(
         }
     }
 
-    if (policy->isTop())
+    /* bug #2550074: "Automatic rules for filter table included twice
+     * in iptables". If user had two policy ruleset objects marked as
+     * "top" rule set, then automaitc rules were added twice. Since we
+     * add rules to automatic_rules_stream only in this one place, it
+     * is sufficient to check if the stream is empty to avoid
+     * duplication.  Note that on windows tellp() seems to return -1
+     * if no data has ever been written to the stream.
+     */
+    long auto_rules_stream_position = automatic_rules_stream.tellp();
+
+    if (policy->isTop() && auto_rules_stream_position <= 0)
     {
-        reset_rules_stream
+        automatic_rules_stream
             << "# ================ Table 'filter', automatic rules"
             << "\n";
-        reset_rules_stream << c.flushAndSetDefaultPolicy();
+        automatic_rules_stream << c.flushAndSetDefaultPolicy();
+
+        if (!prolog_done && prolog_place == "after_flush" &&
+            !fw->getOptionsObject()->getBool("use_iptables_restore"))
+        {
+            automatic_rules_stream << "prolog_commands" << endl << endl;
+            prolog_done = true;
+        }
+
+        automatic_rules_stream << c.printAutomaticRules();
         empty_output = false;
     }
 
@@ -794,7 +799,7 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         std::map<const std::string, bool> minus_n_commands_nat;
 
 
-        vector<bool> ipv4_6_runs;
+        vector<int> ipv4_6_runs;
         string generated_script;
 
         findImportedRuleSets(fw, all_policies);
@@ -808,20 +813,21 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
         if (options->getStr("ipv4_6_order").empty() ||
             options->getStr("ipv4_6_order") == "ipv4_first")
         {
-            if (ipv4_run) ipv4_6_runs.push_back(false);
-            if (ipv6_run) ipv4_6_runs.push_back(true);
+            if (ipv4_run) ipv4_6_runs.push_back(AF_INET);
+            if (ipv6_run) ipv4_6_runs.push_back(AF_INET6);
         }
 
         if (options->getStr("ipv4_6_order") == "ipv6_first")
         {
-            if (ipv6_run) ipv4_6_runs.push_back(true);
-            if (ipv4_run) ipv4_6_runs.push_back(false);
+            if (ipv6_run) ipv4_6_runs.push_back(AF_INET6);
+            if (ipv4_run) ipv4_6_runs.push_back(AF_INET);
         }
 
-        for (vector<bool>::iterator i=ipv4_6_runs.begin(); 
+        for (vector<int>::iterator i=ipv4_6_runs.begin(); 
              i!=ipv4_6_runs.end(); ++i)
         {
-            bool ipv6_policy = *i;
+            int policy_af = *i;
+            bool ipv6_policy = (policy_af == AF_INET6);
 
             /*
               clear chain tracker map only between ipv4/ipv6 runs
@@ -847,16 +853,14 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
                  p!=all_nat.end(); ++p)
             {
                 NAT *nat = NAT::cast(*p);
-                if (nat->isV6()==ipv6_policy) nat_count++;
+                if (nat->matchingAddressFamily(policy_af)) nat_count++;
             }
 
-            
-
             for (list<FWObject*>::iterator p=all_policies.begin();
                  p!=all_policies.end(); ++p)
             {
                 Policy *policy = Policy::cast(*p);
-                if (policy->isV6()==ipv6_policy) policy_count++;
+                if (policy->matchingAddressFamily(policy_af)) policy_count++;
             }
 
             if (nat_count || policy_count)
@@ -867,10 +871,10 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
                 prep->compile();
             }
 
-            ostringstream reset_rules;
-            ostringstream c_str;
-            ostringstream m_str;
-            ostringstream n_str;
+            ostringstream automaitc_rules_stream;
+            ostringstream filter_rules_stream;
+            ostringstream mangle_rules_stream;
+            ostringstream nat_rules_stream;
             bool empty_output = true;
             //MangleTableCompiler_ipt *top_level_mangle_compiler = NULL;
 
@@ -881,14 +885,14 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
                 assignRuleSetChain(nat);
                 string branch_name = nat->getName();
                 
-                if (nat->isV6()!=ipv6_policy) continue;
+                if (!nat->matchingAddressFamily(policy_af)) continue;
 
                 // compile NAT rules before policy rules because policy
                 // compiler needs to know the number of virtual addresses
                 // being created for NAT
                 NATCompiler_ipt n(
-                    objdb, fwobjectname.toUtf8().constData(), ipv6_policy, oscnf,
-                    &minus_n_commands_nat);
+                    objdb, fwobjectname.toUtf8().constData(), ipv6_policy,
+                    oscnf, &minus_n_commands_nat);
                 n.setSourceRuleSet( nat );
                 n.setRuleSetName(branch_name);
 
@@ -908,22 +912,26 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
 
                 if (n.getCompiledScriptLength() > 0)
                 {
-                    n_str << "# ================ Table 'nat', rule set "
-                          << branch_name << "\n";
+                    nat_rules_stream << "# ================ Table 'nat', "
+                                     << " rule set "
+                                     << branch_name << "\n";
 
                     if (n.haveErrorsAndWarnings())
                     {
-                        n_str << "# NAT compiler errors and warnings:"
-                              << "\n";
-                        n_str << n.getErrors("# ");
+                        nat_rules_stream << "# NAT compiler errors and "
+                                         << "warnings:\n";
+                        nat_rules_stream << n.getErrors("# ");
                     }
 
                     if (nat->isTop())
-                        n_str << n.flushAndSetDefaultPolicy();
+                    {
+                        nat_rules_stream << n.flushAndSetDefaultPolicy();
+                        nat_rules_stream << n.printAutomaticRules();
+                    }
 
-                    n_str << n.getCompiledScript();
-                    //n_str << n.commit();
-                    n_str << "\n";
+                    nat_rules_stream << n.getCompiledScript();
+                    //nat_rules_stream << n.commit();
+                    nat_rules_stream << "\n";
                     empty_output = false;
                 }
             }
@@ -932,13 +940,13 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
                  p!=all_policies.end(); ++p )
             {
                 bool empty_policy = processPolicyRuleSet(
-                    fwobjectname,
+                    fw,
                     *p,
-                    c_str,
-                    m_str,
-                    reset_rules,
+                    filter_rules_stream,
+                    mangle_rules_stream,
+                    automaitc_rules_stream,
                     oscnf,
-                    ipv6_policy,
+                    policy_af,
                     minus_n_commands_filter,
                     minus_n_commands_mangle);
 
@@ -961,10 +969,10 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
             }
 
             generated_script += dumpScript(nocomm, fw,
-                                           reset_rules.str(),
-                                           n_str.str(),
-                                           m_str.str(),
-                                           c_str.str(),
+                                           automaitc_rules_stream.str(),
+                                           nat_rules_stream.str(),
+                                           mangle_rules_stream.str(),
+                                           filter_rules_stream.str(),
                                            ipv6_policy);
         }
 
@@ -1075,26 +1083,34 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
  * in firewall options.
  */
         script << oscnf->printPathForAllTools(DISTRO);
+        script << oscnf->printShellFunctions(nocomm);
 
-        string prolog_place= fw->getOptionsObject()->getStr("prolog_place");
+        string prolog_place = fw->getOptionsObject()->getStr("prolog_place");
         if (prolog_place == "") prolog_place="top";
 
-        if (prolog_place == "top")
+        /* there is no way to stick prolog commands between iptables
+         * reset and iptables rules if we use iptables-restore to
+         * activate policy. Therefore, if prolog needs to be ran after
+         * iptables flush and we use iptables-restore, we run prolog
+         * on top of the script.
+         */
+        if (!prolog_done &&
+            (prolog_place == "top" ||
+             (prolog_place == "after_flush" && 
+              fw->getOptionsObject()->getBool("use_iptables_restore"))))
         {
-            script <<
-                addPrologScript(
-                    nocomm, fw->getOptionsObject()->getStr("prolog_script"));
+            script << "prolog_commands" << endl << endl;
+            prolog_done = true;
         }
 
 	script << oscnf->getCompiledScript();
 
         script << "\n";
 
-        if (prolog_place == "after_interfaces")
+        if (!prolog_done && prolog_place == "after_interfaces")
         {
-            script <<
-                addPrologScript(
-                    nocomm, fw->getOptionsObject()->getStr("prolog_script"));
+            script << "prolog_commands" << endl << endl;
+            prolog_done = true;
         }
 
         script << "log '";
@@ -1115,41 +1131,30 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
                << user_name;
         }
 
-        script << "'" << "\n";
-
-	script << "\n";
+        script << "'" << endl;
+	script << endl;
 
         script << generated_script;
 
         script << r.getCompiledScript();
 
-        oscnf->epilog();
 	script << oscnf->getCompiledScript();
 
-        if ( !nocomm )
-        {
-            script << "\n";
-            script << "#" << "\n";
-            script << "# Epilog script" << "\n";
-            script << "#" << "\n";
-        }
+	script << endl;
+        script << "epilog_commands" << endl;
 
-        string post_hook= fw->getOptionsObject()->getStr("epilog_script");
-        script << post_hook << "\n";
+        script << oscnf->printIPForwardingCommands(nocomm);
+        script << endl;
 
-        if ( !nocomm )
-        {
-            script << "\n";
-            script << "# End of epilog script" << "\n";
-            script << "#" << "\n";
-        }
+        // no need to do this because we now abort the script if
+        // iptables-restore returned an error and exit with the same
+        // error code. This means we can only get to this point in
+        // the script if iptables-restore returned "0"
+        //
+        // if (options->getBool("use_iptables_restore"))
+        //     script << "exit $IPTABLES_RESTORE_RES";
 
-        script << "\n";
-
-        if (options->getBool("use_iptables_restore"))
-            script << "exit $IPTABLES_RESTORE_RES";
-
-	script << "\n";
+	script << endl;
 
         QFile fw_file(fw_file_name);
         if (fw_file.open(QIODevice::WriteOnly))
@@ -1158,10 +1163,13 @@ _("Dynamic interface %s should not have an IP address object attached to it. Thi
             fw_str << script_buffer;
             fw_file.close();
             fw_file.setPermissions(QFile::ReadOwner | QFile::WriteOwner |
-                                   QFile::ReadGroup | QFile::ReadOther);
+                                   QFile::ReadGroup | QFile::ReadOther |
+                                   QFile::ExeOwner | 
+                                   QFile::ExeGroup |
+                                   QFile::ExeOther );
         }
 
-        cout << _(" Compiled successfully") << std::endl << flush;
+        cout << " Compiled successfully" << std::endl << flush;
         
         return 0;
 
