@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_pf.h 312 2008-07-05 17:40:12Z vadim $
+  $Id: NATCompiler_pf.h 793 2009-03-26 04:07:26Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -343,7 +343,8 @@ namespace fwcompiler {
             std::string                      current_rule_label;
 
             virtual void _printProtocol(libfwbuilder::Service *srv);
-            virtual void _printPort(libfwbuilder::Service *srv,bool print_range_end);
+            virtual void _printPort(libfwbuilder::Service *srv,
+                                    bool lhs);
 
             virtual void _printAddrList(libfwbuilder::FWObject  *o,bool negflag);
             virtual void _printREAddr(libfwbuilder::RuleElement *o);
