@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: NATCompiler_pf_writers.cpp 312 2008-07-05 17:40:12Z vadim $
+  $Id: NATCompiler_pf_writers.cpp 793 2009-03-26 04:07:26Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -126,20 +126,22 @@ bool NATCompiler_pf::PrintRule::processNext()
 
     char errstr[1024];
 
-    if (osrc==NULL ||
-        odst==NULL ||
-        osrv==NULL ||
-        tsrc==NULL ||
-        tdst==NULL ||
-        tsrv==NULL) 
+    if (osrc==NULL || odst==NULL || osrv==NULL ||
+        tsrc==NULL || tdst==NULL || tsrv==NULL) 
     {
-        if (osrc==NULL)sprintf(errstr, "NAT rule %s: osrc==NULL", rule->getLabel().c_str() );
-        if (odst==NULL)sprintf(errstr, "NAT rule %s: odst==NULL", rule->getLabel().c_str() );
-        if (osrv==NULL)sprintf(errstr, "NAT rule %s: osrv==NULL", rule->getLabel().c_str() );
+        if (osrc==NULL)
+            sprintf(errstr,"NAT rule %s: osrc==NULL", rule->getLabel().c_str());
+        if (odst==NULL)
+            sprintf(errstr,"NAT rule %s: odst==NULL", rule->getLabel().c_str());
+        if (osrv==NULL)
+            sprintf(errstr,"NAT rule %s: osrv==NULL", rule->getLabel().c_str());
 
-        if (tsrc==NULL)sprintf(errstr, "NAT rule %s: tsrc==NULL", rule->getLabel().c_str() );
-        if (tdst==NULL)sprintf(errstr, "NAT rule %s: tdst==NULL", rule->getLabel().c_str() );
-        if (tsrv==NULL)sprintf(errstr, "NAT rule %s: tsrv==NULL", rule->getLabel().c_str() );
+        if (tsrc==NULL)
+            sprintf(errstr,"NAT rule %s: tsrc==NULL", rule->getLabel().c_str());
+        if (tdst==NULL)
+            sprintf(errstr,"NAT rule %s: tdst==NULL", rule->getLabel().c_str());
+        if (tsrv==NULL)
+            sprintf(errstr,"NAT rule %s: tsrv==NULL", rule->getLabel().c_str());
 
         compiler->abort(errstr);
     }
@@ -228,25 +230,46 @@ bool NATCompiler_pf::PrintRule::processNext()
 
 void NATCompiler_pf::PrintRule::_printProtocol(Service *srv)
 {
-    if ( ! CustomService::isA(srv) && ! TagService::isA(srv))
+    // CustomService returns protocol name starting with v3.0.4
+    if (CustomService::isA(srv))
     {
-        string s=srv->getProtocolName();
-        if (s=="ip") s="{tcp udp icmp}";
+        // check if the code string for this custom service already includes
+        // "proto ..." fragment
+        string code = CustomService::cast(srv)->getCodeForPlatform(
+            compiler->myPlatformName());
+        std::size_t minus_p = code.find("proto ");
+        if (minus_p != string::npos) return;
+    }
+
+    if ( !TagService::isA(srv))
+    {
+        string s = srv->getProtocolName();
+        if (s=="ip" || s=="any") s="{tcp udp icmp}";
         compiler->output << "proto " <<  s << " ";
     }
 }
 
-void NATCompiler_pf::PrintRule::_printPort(Service *srv, bool print_range_end)
+/*
+ * print port numbers for the service. For most platforms that inherit
+ * classes for PF this is sufficient, but PF itself also supports
+ * TagService and this method prints "tagged" keyword for it as well.
+ * Arg <lhs> controls which side of the "->" this service is on.  On
+ * the right hand side PF supports shortcut spec 'NNNN:*', but it is
+ * not allowed on the left hand side. Also keyword "tagged" is only
+ * allowed on the left hand side of "->".
+ */
+void NATCompiler_pf::PrintRule::_printPort(Service *srv, bool lhs)
 {
-    if (TCPService::isA(srv) || UDPService::isA(srv)) {
-	int drs=TCPUDPService::cast(srv)->getDstRangeStart();
-	int dre=TCPUDPService::cast(srv)->getDstRangeEnd();
+    if (TCPService::isA(srv) || UDPService::isA(srv))
+    {
+	int drs = TCPUDPService::cast(srv)->getDstRangeStart();
+	int dre = TCPUDPService::cast(srv)->getDstRangeEnd();
 	if (drs!=0)
         {
             compiler->output << "port " << drs;
             if (dre!=0 && dre!=drs)
             {
-                if (print_range_end)
+                if (lhs)
                     compiler->output << ":" << dre;
                 else
                     compiler->output << ":*";
@@ -254,13 +277,15 @@ void NATCompiler_pf::PrintRule::_printPort(Service *srv, bool print_range_end)
         }
         compiler->output  << " ";
     }
-    if (TagService::isA(srv)) 
+
+    if (lhs && TagService::isA(srv)) 
     {
-	compiler->output << "tagged " << TagService::constcast(srv)->getCode() << " ";
+	compiler->output << "tagged "
+                         << TagService::constcast(srv)->getCode() << " ";
     }
 }
 
-void NATCompiler_pf::PrintRule::_printNegation(libfwbuilder::RuleElement  *rel)
+void NATCompiler_pf::PrintRule::_printNegation(RuleElement  *rel)
 {
     if (rel->getNeg())
 	compiler->output << "! ";
