@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf.cpp 545 2008-09-19 20:27:58Z vadim $
+  $Id: PolicyCompiler_pf.cpp 709 2008-12-25 20:38:53Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -216,7 +216,7 @@ bool PolicyCompiler_pf::processMultiAddressObjectsInRE::processNext()
 
         for (FWObject::iterator i=maddr_runtime.begin(); i!=maddr_runtime.end(); i++)
         {
-            PolicyRule *r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+            PolicyRule *r= compiler->dbcopy->createPolicyRule();
             compiler->temp_ruleset->add(r);
             r->duplicate(rule);
             nre=RuleElement::cast( r->getFirstByType(re_type) );
@@ -266,7 +266,7 @@ bool PolicyCompiler_pf::splitIfFirewallInSrc::processNext()
 
 	    RuleElementSrc *nsrc;
 
-	    r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	    r= compiler->dbcopy->createPolicyRule();
 	    compiler->temp_ruleset->add(r);
 	    r->duplicate(rule);
 	    nsrc=r->getSrc();
@@ -311,7 +311,7 @@ bool PolicyCompiler_pf::splitIfFirewallInDst::processNext()
 
 	    RuleElementDst *ndst;
 
-	    r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	    r= compiler->dbcopy->createPolicyRule();
 	    compiler->temp_ruleset->add(r);
 	    r->duplicate(rule);
 	    ndst=r->getDst();
@@ -349,29 +349,23 @@ bool PolicyCompiler_pf::fillDirection::processNext()
  */
     if (rule->getDirectionAsString()=="" || rule->getInterfaceId()==-1 )
     {
-	if ( compiler->getCachedFwOpt()->getBool("pass_all_out") )
-        {
-            if (!rule->isFallback()) rule->setDirection( PolicyRule::Inbound );
-	} else 
-        {
-	    rule->setDirection( PolicyRule::Both );
+        rule->setDirection( PolicyRule::Both );
 
-	    Address  *src = compiler->getFirstSrc(rule);
-	    Address  *dst = compiler->getFirstDst(rule);
-	    //int fwid = compiler->getFwId();
+        Address  *src = compiler->getFirstSrc(rule);
+        Address  *dst = compiler->getFirstDst(rule);
+        //int fwid = compiler->getFwId();
 
-            if (src==NULL || dst==NULL)
-                compiler->abort("Broken src or dst in rule "+rule->getLabel());
+        if (src==NULL || dst==NULL)
+            compiler->abort("Broken src or dst in rule "+rule->getLabel());
 
-            if (!src->isAny() && !dst->isAny() &&
-                compiler->complexMatch(compiler->fw, src) &&
-                compiler->complexMatch(compiler->fw, dst)) return true;
+        if (!src->isAny() && !dst->isAny() &&
+            compiler->complexMatch(compiler->fw, src) &&
+            compiler->complexMatch(compiler->fw, dst)) return true;
 
-            if (!src->isAny() && compiler->complexMatch(compiler->fw, src))
-                rule->setDirection( PolicyRule::Outbound );
-            if (!dst->isAny() && compiler->complexMatch(compiler->fw, dst))
-                rule->setDirection( PolicyRule::Inbound );
-	}
+        if (!src->isAny() && compiler->complexMatch(compiler->fw, src))
+            rule->setDirection( PolicyRule::Outbound );
+        if (!dst->isAny() && compiler->complexMatch(compiler->fw, dst))
+            rule->setDirection( PolicyRule::Inbound );
     }
     return true;
 }
@@ -388,8 +382,7 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
              !getCachedFwOpt()->getStr("mgmt_addr").empty() )
         {
             PolicyRule *r;
-            TCPService *ssh =
-                TCPService::cast(dbcopy->create(TCPService::TYPENAME) );
+            TCPService *ssh = dbcopy->createTCPService();
             ssh->setDstRangeStart(22);
             ssh->setDstRangeEnd(22);
 
@@ -428,8 +421,7 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
                 abort( errstr );
             }
 
-            Network *mgmt_workstation =
-                Network::cast(dbcopy->create(Network::TYPENAME));
+            Network *mgmt_workstation = dbcopy->createNetwork();
             mgmt_workstation->setName("mgmt_addr");
             mgmt_workstation->setAddress( addr );
             mgmt_workstation->setNetmask( netmask );
@@ -439,7 +431,7 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
             cacheObj(mgmt_workstation); // to keep cache consistent
 
 
-            r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME) );
+            r= dbcopy->createPolicyRule();
             temp_ruleset->add(r);
             r->setAction(PolicyRule::Accept);
             r->setLogging(false);
@@ -466,55 +458,20 @@ void PolicyCompiler_pf::addDefaultPolicyRule()
             combined_ruleset->push_front(r);
         }
 
-        if ( getCachedFwOpt()->getBool("pass_all_out") )
-        {
-            PolicyRule *r;
-            FWOptions *ruleopt;
-
-            r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME) );
-            temp_ruleset->add(r);
-            r->setAction(PolicyRule::Accept);
-            r->setLogging( getCachedFwOpt()->getBool("fallback_log") );
-            r->setDirection(PolicyRule::Outbound);
-            r->setPosition(10000);
-            r->setComment("   fallback rule ");
-            r->setHidden(true);
-            r->setFallback(true);
-            r->setLabel("fallback rule");
-            combined_ruleset->push_back(r);
-
-            r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME) );
-            temp_ruleset->add(r);
-            r->setAction(PolicyRule::Deny);
-            r->setLogging( getCachedFwOpt()->getBool("fallback_log") );
-            r->setDirection(PolicyRule::Inbound);
-            r->setPosition(10001);
-            r->setComment("   fallback rule ");
-            r->setHidden(true);
-            r->setFallback(true);
-            r->setLabel("fallback rule");
-            ruleopt = r->getOptionsObject();
-            ruleopt->setBool("stateless", true);
-            combined_ruleset->push_back(r);
-        } else
-        {
-            PolicyRule *r=
-                PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME) );
-            FWOptions *ruleopt;
-
-            temp_ruleset->add(r);
-            r->setAction(PolicyRule::Deny);
-            r->setLogging( getCachedFwOpt()->getBool("fallback_log") );
-            r->setDirection(PolicyRule::Both);
-            r->setPosition(10000);
-            r->setComment("   fallback rule ");
-            r->setHidden(true);
-            r->setFallback(true);
-            r->setLabel("fallback rule");
-            ruleopt = r->getOptionsObject();
-            ruleopt->setBool("stateless", true);
-            combined_ruleset->push_back(r);
-        }
+        PolicyRule *r = dbcopy->createPolicyRule();
+        FWOptions *ruleopt;
+        temp_ruleset->add(r);
+        r->setAction(PolicyRule::Deny);
+        r->setLogging( getCachedFwOpt()->getBool("fallback_log") );
+        r->setDirection(PolicyRule::Both);
+        r->setPosition(10000);
+        r->setComment("   fallback rule ");
+        r->setHidden(true);
+        r->setFallback(true);
+        r->setLabel("fallback rule");
+        ruleopt = r->getOptionsObject();
+        ruleopt->setBool("stateless", true);
+        combined_ruleset->push_back(r);
     }
 }
 
@@ -546,15 +503,13 @@ bool PolicyCompiler_pf::SplitDirection::processNext()
     PolicyRule *rule=getNext(); if (rule==NULL) return false;
 
     if (rule->getDirection()==PolicyRule::Both) {
-	PolicyRule *r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	PolicyRule *r= compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	r->setDirection(PolicyRule::Inbound);
 	tmp_queue.push_back(r);
 
-	r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	r= compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	r->setDirection(PolicyRule::Outbound);
@@ -584,8 +539,7 @@ bool PolicyCompiler_pf::ProcessScrubOption::processNext()
 	    return true;
 	}
 
-	PolicyRule *r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	PolicyRule *r = compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	r->setAction(PolicyRule::Scrub);
@@ -605,8 +559,7 @@ bool PolicyCompiler_pf::ProcessScrubOption::processNext()
     if ( (srv->getBool("short_fragm") || srv->getBool("fragm")) &&
 	 ( rule->getAction()==PolicyRule::Deny || rule->getAction()==PolicyRule::Reject) ) {
 
-	PolicyRule *r= PolicyRule::cast(
-	    compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	PolicyRule *r = compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
 	r->setAction(PolicyRule::Scrub);
@@ -645,18 +598,20 @@ bool PolicyCompiler_pf::doSrcNegation::processNext()
         RuleElementSrc *nsrc;
 	PolicyRule     *r;
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	r= compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
-	if (rule->getAction()==PolicyRule::Accept)  r->setAction(PolicyRule::Deny);
-	else	                                    r->setAction(PolicyRule::Accept);
+	if (rule->getAction()==PolicyRule::Accept)
+            r->setAction(PolicyRule::Deny);
+	else
+            r->setAction(PolicyRule::Accept);
         nsrc=r->getSrc();
         nsrc->setNeg(false);
 	r->setBool("quick",true);
         r->setLogging(false);
 	tmp_queue.push_back(r);
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	r= compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
         nsrc=r->getSrc();
@@ -682,18 +637,20 @@ bool PolicyCompiler_pf::doDstNegation::processNext()
         RuleElementDst *ndst;
 	PolicyRule     *r;
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	r= compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
-	if (rule->getAction()==PolicyRule::Accept)  r->setAction(PolicyRule::Deny);
-	else	                                    r->setAction(PolicyRule::Accept);
+	if (rule->getAction()==PolicyRule::Accept)
+            r->setAction(PolicyRule::Deny);
+	else
+            r->setAction(PolicyRule::Accept);
         ndst=r->getDst();
         ndst->setNeg(false);
 	r->setBool("quick",true);
         r->setLogging(false);
 	tmp_queue.push_back(r);
 
-	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+	r= compiler->dbcopy->createPolicyRule();
 	compiler->temp_ruleset->add(r);
 	r->duplicate(rule);
         ndst=r->getDst();
@@ -853,19 +810,21 @@ bool PolicyCompiler_pf::splitIfInterfaceInRE::processNext()
     {
         RuleElement *nre;
 
-        PolicyRule *r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+        PolicyRule *r= compiler->dbcopy->createPolicyRule();
         compiler->temp_ruleset->add(r);
         r->duplicate(rule);
         nre=RuleElement::cast( r->getFirstByType(re_type) );
         nre->clearChildren();
-        for (FWObject::iterator i=cl.begin(); i!=cl.end(); i++) nre->addRef( *i );
+        for (FWObject::iterator i=cl.begin(); i!=cl.end(); i++)
+            nre->addRef( *i );
         tmp_queue.push_back(r);
 
-        r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+        r= compiler->dbcopy->createPolicyRule();
         compiler->temp_ruleset->add(r);
         r->duplicate(rule);
         nre=RuleElement::cast( r->getFirstByType(re_type) );
-        for (FWObject::iterator i=cl.begin(); i!=cl.end(); i++) nre->removeRef( *i );
+        for (FWObject::iterator i=cl.begin(); i!=cl.end(); i++)
+            nre->removeRef( *i );
         tmp_queue.push_back(r);
 
         return true;
@@ -942,8 +901,7 @@ bool PolicyCompiler_pf::separateServiceObject::processNext()
 
 	if (condition(s))
         {
-            PolicyRule *r= PolicyRule::cast(
-                compiler->dbcopy->create(PolicyRule::TYPENAME) );
+            PolicyRule *r = compiler->dbcopy->createPolicyRule();
             compiler->temp_ruleset->add(r);
             r->duplicate(rule);
             RuleElementSrv *nsrv=r->getSrv();
