@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_pf_writers.cpp 555 2008-09-26 18:58:17Z vadim $
+  $Id: PolicyCompiler_pf_writers.cpp 790 2009-03-24 07:31:48Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -124,7 +124,11 @@ void PolicyCompiler_pf::PrintRule::_printAction(PolicyRule *rule)
         if (ruleset==NULL)
             compiler->abort(string("Branching rule ") + rule->getLabel() +
                             " refers ruleset that does not exist");
-        compiler->output << "anchor " << ruleset->getName() << " ";
+        string ruleset_name = ruleset->getName();
+        if (ruleset_name.find("/*")!=string::npos)
+            compiler->output << "anchor \"" << ruleset_name << "\" ";
+        else
+            compiler->output << "anchor " << ruleset_name << " ";
         break;
     }
     default:
@@ -410,13 +414,26 @@ void PolicyCompiler_pf::PrintRule::_printAF(PolicyRule*)
     else compiler->output << "inet ";
 }
 
-void PolicyCompiler_pf::PrintRule::_printProtocol(libfwbuilder::Service *srv)
+void PolicyCompiler_pf::PrintRule::_printProtocol(Service *srv)
 {
+    // CustomService returns protocol name starting with v3.0.4
+    // However CustomService can return protocol name "any", which we should
+    // just skip.
 
-    if (!srv->isAny() &&
-        !CustomService::isA(srv) &&
-        !TagService::isA(srv) && 
-        !UserService::isA(srv) && 
+    // CustomService returns protocol name starting with v3.0.4
+    if (CustomService::isA(srv))
+    {
+        // check if the code string for this custom service already includes
+        // "proto ..." fragment
+        string code = CustomService::cast(srv)->getCodeForPlatform(
+            compiler->myPlatformName());
+        std::size_t minus_p = code.find("proto ");
+        if (minus_p != string::npos) return;
+        string pn = srv->getProtocolName();
+        if (pn == "any") return;
+    }
+
+    if (!srv->isAny() && !TagService::isA(srv) && !UserService::isA(srv) && 
         srv->getProtocolName()!="ip")
     {
 	compiler->output << "proto ";
