@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Preprocessor_pf.cpp,v 1.1 2006/05/21 04:47:10 vkurland Exp $
+  $Id: Preprocessor_pf.cpp 473 2008-08-18 01:47:56Z vadim $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -36,10 +36,5 @@ using namespace std;
 void Preprocessor_pf::convertObject(FWObject *obj)
 {
     Preprocessor::convertObject(obj);
-
-    AddressTable *adt = AddressTable::cast(obj);
-    if (adt!=NULL && adt->isCompileTime())
-        adt->setRunTime(true);
-
 }
 
